@@ -77,13 +77,16 @@ setMethod("convpow",
 
 setMethod("convpow",
           signature(D1 = "LatticeDistribution"),
-          function(D1, N){
+          function(D1, N, ep = getdistrOption("TruncQuantile")){
             if( !.isNatural0(N))
               stop("N has to be a natural (or 0)")
             if (N==0) return(Dirac(0))
-
             if (N==1) return(D1)
 
+            if(!is.numeric(ep)) stop("argument 'ep' must be a numeric.")
+            if(length(ep)!=1) stop("argument 'ep' must be a numeric of length 1.")
+            if((ep<0)||(ep>1)) stop("argument 'ep' must be in (0,1).")
+
             w <- width(lattice(D1))
 
             supp0 <- support(D1)
@@ -98,18 +101,23 @@ setMethod("convpow",
             newd <- Re(fft(ftde1^N, inverse = TRUE)) / length(ftde1)
             newd <- (abs(newd) >= .Machine$double.eps)*newd
 
-            rsum.u <- min( sum( rev(cumsum(rev(newd))) <=
-                                getdistrOption("TruncQuantile")/2)+1,
-                           length(supp1))
-            rsum.l <- max( sum( cumsum(newd) <
-                                getdistrOption("TruncQuantile")/2),
-                          1)
+            rsum.u <- min( sum( rev(cumsum(rev(newd))) <= ep/2)+1, length(supp1))
+            rsum.l <- max( sum( cumsum(newd) < ep/2), 1)
 
             newd <- newd[rsum.l:rsum.u]
             newd <- newd/sum(newd)
             supp1 <- supp1[rsum.l:rsum.u]
-
-            return(LatticeDistribution(supp=supp1,prob=newd))
+            
+            supp2 <- supp1[newd>ep]
+            newd2 <- newd[newd>ep]
+            newd2 <- newd2/sum(newd2)
+            
+            if( length(supp1) >= 2 * length(supp2))
+               return(DiscreteDistribution(supp = supp2, prob = newd2,
+                                           .withArith = TRUE))
+            else  
+               return(LatticeDistribution(supp = supp1, prob = newd,
+                                          .withArith = TRUE))
 })
 
 ###############################################################################
@@ -120,7 +128,7 @@ setMethod("convpow",
 #
 setMethod("convpow",
           signature(D1 = "AcDcLcDistribution"),
-          function(D1, N){
+          function(D1, N, ep = getdistrOption("TruncQuantile")){
             if( !.isNatural0(N))
               stop("N has to be a natural (or 0)")
             if (N==0) return(Dirac(0))
@@ -129,6 +137,10 @@ setMethod("convpow",
         if(is(e1,"DiscreteDistribution")) return(convpow(e1,N))
         if(is(e1,"AbscontDistribution")) return(convpow(e1,N))
 
+            if(!is.numeric(ep)) stop("argument 'ep' must be a numeric.")
+            if(length(ep)!=1) stop("argument 'ep' must be a numeric of length 1.")
+            if((ep<0)||(ep>1)) stop("argument 'ep' must be in (0,1).")
+
         aw1 <- acWeight(e1)
         dw1 <- 1-aw1
         dD1 <- discretePart(e1)
@@ -136,32 +148,39 @@ setMethod("convpow",
         dD1 <- discretePart(e1)
         if(is(dD1,"LatticeDistribution"))
            dD1 <- as(dD1,"LatticeDistribution")
-        dDm <- max(d.discrete(e1)(support(e1)))*dw1
-
-        ep <- getdistrOption("TruncQuantile")
+  #      dDm <- max(d.discrete(e1)(support(e1)))*dw1
 
         if(aw1<ep) return(convpow(dD1,N))
-        if(1-aw1<ep) return(convpow(aD1,N))
+        if(dw1<ep) return(convpow(aD1,N))
 
-        maxN <- ceiling(2*log(ep)/log(dDm))
+        maxN <- ceiling(2*log(ep)/log(dw1))
         Nm <- min(maxN,N)
         Mm <- N%/%Nm
         Rm <- N-Mm*Nm
-        print(maxN)
-
+   
         sumM <- function(mm){
-                DList <- lapply(seq(mm+1)-1,
-                               function(x) {
+                db <- dbinom(0:mm, size = mm, prob = aw1)                
+                im <- (0:mm)[db>ep^2]
+                db <- db[db>ep^2]
+                db <- db/sum(db)
+                if(length(im)>1){
+                      DList <- lapply(im,
+                                function(x) {
                                    S.a <- convpow(aD1, x)
                                    S.d <- convpow(dD1, mm-x) #as(dD1,
                                           #  "DiscreteDistribution"), mm-x)
                                    as(S.a+S.d,"UnivarLebDecDistribution")
-                               })
-                erg <- do.call(flat.LCD, c(DList,
-                        alist(mixCoeff = dbinom(0:mm, size = mm, prob = aw1))))
-                erg}
+                               }) 
+                      erg <- do.call(flat.LCD, c(DList, alist(mixCoeff = db)))
+                }else{
+                      DList <- as(convpow(aD1,im)+convpow(dD1,mm-im),"UnivarLebDecDistribution")           
+                      erg <- flat.LCD(DList, mixCoeff = 1)
+                      } 
+                return(erg)
+        }
+        
         erg <- sumM(Nm)
-        if(Mm>1) erg <- convpow(erg,Mm)
+        if(Mm>1) erg <- convpow(erg,Mm,ep=ep)
         if(Rm>0) erg <- sumM(Rm)+ as(erg,"UnivarLebDecDistribution")
         if(is(erg,"UnivarLebDecDistribution")) erg <- simplifyD(erg)
         return(erg)
@@ -176,9 +195,10 @@ setMethod("convpow",
             if (N==0) return(Dirac(0))
             if (N==1) return(D1)
             if (N==2) return(D1+D1)
-            D11 <- if (N%%2==1) D1 else Dirac(0)
             DN1 <- convpow(D1,N%/%2)
-            return((DN1+DN1)+D11)
+            DN1 <- DN1 + DN1
+            if (N%%2==1) DN1 <- DN1+D1 
+            return(DN1)
             })
 ###############################################################################
             
