@@ -10,6 +10,13 @@ setMethod("*", c("AffLinUnivarLebDecDistribution","numeric"),
           if (isTRUE(all.equal(e2,1))) return(e1)
           if (isTRUE(all.equal(e2,0)))
                return(new("Dirac", location = 0))
+          
+          if(.isEqual(e1@a*e2,1)&&.isEqual(e1@b,0)){
+             obj <- e1@X0
+             if(getdistrOption("simplifyD"))
+                obj <- simplifyD(obj)
+             return(obj)
+          }   
 
           Distr <- UnivarLebDecDistribution(
                      discretePart = discretePart(e1)*e2,
@@ -28,7 +35,10 @@ setMethod("*", c("AffLinUnivarLebDecDistribution","numeric"),
                     q = Distr@q, X0 = e1@X0, mixDistr = Distr@mixDistr,
                     mixCoeff = Distr@mixCoeff,
                     a = e1@a*e2, b = e1@b, .withSim  = e1@.withSim,
-                    .withArith = TRUE)
+                    .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
+                     gaps = gaps(Distr), support = support(Distr)
+                     )
           object})
 
 setMethod("+", c("AffLinUnivarLebDecDistribution","numeric"),
@@ -36,6 +46,13 @@ setMethod("+", c("AffLinUnivarLebDecDistribution","numeric"),
           if (length(e2)>1) stop("length of operator must be 1")
           if (isTRUE(all.equal(e2,0))) return(e1)
 
+          if(.isEqual(e1@a,1)&&.isEqual(e1@b+e2,0)){
+             obj <- e1@X0
+             if(getdistrOption("simplifyD"))
+                obj <- simplifyD(obj)
+             return(obj)
+          }   
+          
           Distr <- UnivarLebDecDistribution(
                      discretePart = discretePart(e1)+e2,
                      acPart = acPart(e1)+e2,
@@ -54,7 +71,10 @@ setMethod("+", c("AffLinUnivarLebDecDistribution","numeric"),
                     q = Distr@q, X0 = e1@X0, mixDistr = Distr@mixDistr,
                     mixCoeff = Distr@mixCoeff,
                     a = e1@a, b = e1@b+e2, .withSim  = e1@.withSim,
-                    .withArith = TRUE)
+                    .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
+                     gaps = gaps(Distr), support = support(Distr)
+                     )
           object})
 
 
